#ifndef __fastrak_h_
#define __fastrak_h_


#include "matrix.h"

/* define's */

#define MAX_STATIONS 4

#define POSITION_FLAG (0x1)
#define ROTATION_FLAG (0x2)
#define XDIRCOS_FLAG (0x4)
#define YDIRCOS_FLAG (0x8)
#define ZDIRCOS_FLAG (0x10)
#define RELATIVE_POSITION_FLAG (0x20)

#define THIS_CONFIG (ROTATION_FLAG | RELATIVE_POSITION_FLAG)
#define NVALUES 6
static char output_format[]={"3,4,1"};

#define BYTES_PER_STATION (3+(4*NVALUES)+2)

/* structs */

typedef struct station_info
{
    int station_num;
    int data_is_valid;
    unsigned int flags;
    float pos[3],rot[3];
    float delta_pos[3];
    float xdircos[3],ydircos[3],zdircos[3];
} station_info;

#define XAXIS 0
#define YAXIS 1
#define ZAXIS 2

#define AZIMUTH 0
#define ELEVATION 1
#define ROLL 2

typedef struct receiver_orientation
{
    /* these values are used directly by the align reference frame
     command.*/
    float xaxis[3],yaxis[3],origin[3];


    int axes[3];
    float rot_scale_sign[3];

    int translation_reassign[3];
    float trans_sign[3];
} receiver_orientation;


/**
  * this stuff maps from the receiver's x,y,z axes and it's azimuth,
  * elevation and roll angles into "user" space.  this one is used for
  * the division 3d mouse.
**/
static receiver_orientation mouse3d =
{

    {200., 0., 00.},
    {00., 200.,0.},
    {0.,0.,0.},

    /* the way that axes[] can be interpreted is as follows:
     axis[2] = XAXIS, which means that "azimuth" events (defined as rotations
     about the receiver's Z axis in the fastrak manual) will be assigned
     to the user's X axis. */
    {YAXIS, ZAXIS,XAXIS},  
    {-1., -1., -1.}, 

    {ZAXIS, XAXIS, YAXIS},
    {-1.,1.,1.}
};

static receiver_orientation vr4 =
{

    {200., 0., 00.},
    {00., 200.,0.},
    {0.,0.,0.},

    /* the way that axes[] can be interpreted is as follows:
     axis[2] = XAXIS, which means that "azimuth" events (defined as rotations
     about the receiver's Z axis in the fastrak manual) will be assigned
     to the user's X axis. */
    {XAXIS, ZAXIS,YAXIS},  
    {1., -1., -1.}, 

    {XAXIS, ZAXIS, YAXIS},
    {-1.,-1.,1.}
};



/* public proc declarations */

/* stuff for fastrak trackers */

int open_fastrak(void);
int reset_fastrak(void);
int close_fastrak(void);
int configure_fastrak_stations(station_info *sa);
int filter_enable(void);
int reset_alignment_frame(int,float *,float *,float *);
int get_fastrak_data(station_info *sa);
int reset_boresight(int station_num, float *new_angles);

/* stuff for division's 3d mouse */
int open_3dmouse(void);
int close_3dmouse(void);


int build_composite_matrix(matrix4x4 *m,float *r);
int recompute_base_rotations(matrix4x4 *mthis, matrix4x4 *mlast, float *azimuth, float *elevation, float *roll);


#endif
